@@ -19,22 +19,54 @@ except ImportError:  # Python2
 
 import django
 from django.conf import settings
+from django.template import loader
 from django.template.context import Context, RequestContext
 from django.utils import six
 
 from .subprocess import check_output
 
+NO_ARGUMENT_OPTIONS = ['--collate', '--no-collate', '-H', '--extended-help', '-g',
+                       '--grayscale', '-h', '--help', '--htmldoc', '--license', '-l',
+                       '--lowquality', '--manpage', '--no-pdf-compression', '-q',
+                       '--quiet', '--read-args-from-stdin', '--readme',
+                       '--use-xserver', '-V', '--version', '--dump-default-toc-xsl',
+                       '--outline', '--no-outline', '--background', '--no-background',
+                       '--custom-header-propagation', '--no-custom-header-propagation',
+                       '--debug-javascript', '--no-debug-javascript', '--default-header',
+                       '--disable-external-links', '--enable-external-links',
+                       '--disable-forms', '--enable-forms', '--images', '--no-images',
+                       '--disable-internal-links', '--enable-internal-links', '-n',
+                       '--disable-javascript', '--enable-javascript', '--keep-relative-links',
+                       '--load-error-handling', '--load-media-error-handling',
+                       '--disable-local-file-access', '--enable-local-file-access',
+                       '--exclude-from-outline', '--include-in-outline', '--disable-plugins',
+                       '--enable-plugins', '--print-media-type', '--no-print-media-type',
+                       '--resolve-relative-links', '--disable-smart-shrinking',
+                       '--enable-smart-shrinking', '--stop-slow-scripts',
+                       '--no-stop-slow-scripts', '--disable-toc-back-links',
+                       '--enable-toc-back-links', '--footer-line', '--no-footer-line',
+                       '--header-line', '--no-header-line', '--disable-dotted-lines',
+                       '--disable-toc-links', '--verbose']
+
 
 def _options_to_args(**options):
-    """Converts ``options`` into a list of command-line arguments."""
+    """
+    Converts ``options`` into a list of command-line arguments.
+    Skip arguments where no value is provided
+    For flag-type (No argument) variables, pass only the name and only then if the value is True
+    """
     flags = []
     for name in sorted(options):
         value = options[name]
-        if value is None:
+        formatted_flag = '--%s' % name if len(name) > 1 else '-%s' % name
+        formatted_flag = formatted_flag.replace('_', '-')
+        accepts_no_arguments = formatted_flag in NO_ARGUMENT_OPTIONS
+        if value is None or (value is False and accepts_no_arguments):
             continue
-        flags.append('--' + name.replace('_', '-'))
-        if value is not True:
-            flags.append(six.text_type(value))
+        flags.append(formatted_flag)
+        if accepts_no_arguments:
+            continue
+        flags.append(six.text_type(value))
     return flags
 
 
@@ -75,6 +107,7 @@ def wkhtmltopdf(pages, output=None, **kwargs):
     if output is None:
         # Standard output.
         output = '-'
+    has_cover = kwargs.pop('has_cover', False)
 
     # Default options:
     options = getattr(settings, 'WKHTMLTOPDF_CMD_OPTIONS', None)
@@ -94,6 +127,10 @@ def wkhtmltopdf(pages, output=None, **kwargs):
     cmd = 'WKHTMLTOPDF_CMD'
     cmd = getattr(settings, cmd, os.environ.get(cmd, 'wkhtmltopdf'))
 
+    # Adding 'cover' option to add cover_file to the pdf to generate.
+    if has_cover:
+        pages.insert(0, 'cover')
+
     ck_args = list(chain(shlex.split(cmd),
                          _options_to_args(**options),
                          list(pages),
@@ -109,19 +146,24 @@ def wkhtmltopdf(pages, output=None, **kwargs):
 
     return check_output(ck_args, **ck_kwargs)
 
-def convert_to_pdf(filename, header_filename=None, footer_filename=None, cmd_options=None):
+def convert_to_pdf(filename, header_filename=None, footer_filename=None, cmd_options=None, cover_filename=None):
     # Clobber header_html and footer_html only if filenames are
     # provided. These keys may be in self.cmd_options as hardcoded
     # static files.
     # The argument `filename` may be a string or a list. However, wkhtmltopdf
     # will coerce it into a list if a string is passed.
     cmd_options = cmd_options if cmd_options else {}
+    if cover_filename:
+        pages = [cover_filename, filename]
+        cmd_options['has_cover'] = True
+    else:
+        pages = [filename]
 
     if header_filename is not None:
         cmd_options['header_html'] = header_filename
     if footer_filename is not None:
         cmd_options['footer_html'] = footer_filename
-    return wkhtmltopdf(pages=filename, **cmd_options)
+    return wkhtmltopdf(pages=pages, **cmd_options)
 
 class RenderedFile(object):
     """
@@ -148,7 +190,8 @@ class RenderedFile(object):
         if self.temporary_file is not None:
             self.temporary_file.close()
 
-def render_pdf_from_template(input_template, header_template, footer_template, context, request=None, cmd_options=None):
+def render_pdf_from_template(input_template, header_template, footer_template, context, request=None, cmd_options=None,
+    cover_template=None):
     # For basic usage. Performs all the actions necessary to create a single
     # page PDF from a single template and context.
     cmd_options = cmd_options if cmd_options else {}
@@ -179,11 +222,19 @@ def render_pdf_from_template(input_template, header_template, footer_template, c
             request=request
         )
         footer_filename = footer_file.filename
+    cover = None
+    if cover_template:
+        cover = RenderedFile(
+            template=cover_template,
+            context=context,
+            request=request
+        )
 
     return convert_to_pdf(filename=input_file.filename,
                           header_filename=header_filename,
                           footer_filename=footer_filename,
-                          cmd_options=cmd_options)
+                          cmd_options=cmd_options,
+                          cover_filename=cover.filename if cover else None)
 
 def content_disposition_filename(filename):
     """
@@ -259,18 +310,21 @@ def make_absolute_paths(content):
 def render_to_temporary_file(template, context, request=None, mode='w+b',
                              bufsize=-1, suffix='.html', prefix='tmp',
                              dir=None, delete=True):
-    if django.VERSION < (1, 8):
-        # If using a version of Django prior to 1.8, ensure ``context`` is an
-        # instance of ``Context``
-        if not isinstance(context, Context):
-            if request:
-                context = RequestContext(request, context)
-            else:
-                context = Context(context)
-        content = template.render(context)
-    else:
-        content = template.render(context, request)
-
+    try:
+        if django.VERSION < (1, 8):
+            # If using a version of Django prior to 1.8, ensure ``context`` is an
+            # instance of ``Context``
+            if not isinstance(context, Context):
+                if request:
+                    context = RequestContext(request, context)
+                else:
+                    context = Context(context)
+            # Handle error when ``request`` is None
+            content = template.render(context)
+        else:
+            content = template.render(context, request)
+    except AttributeError:
+        content = loader.render_to_string(template, context)
     content = smart_text(content)
     content = make_absolute_paths(content)
 
